function LibUtilities(ScreenService, eventAggregator, templateManager, requestDataSourceEngine, restEngine, MessageService, ZhuAclConfig, General, ZHPromise) {
    var self = this;
    this.authenticationContainer = 'login#authentication';
    this.expiredSessionMethod = null;

    var STYLESHEET_TYPE    = 'stylesheet';
    var addLinkTagInHead = function(href, type) {
        var lnk = document.createElement('link');
        lnk.setAttribute('rel', type );
        if(type == STYLESHEET_TYPE) {
            lnk.setAttribute('type', 'text/css');
        }
        lnk.setAttribute('href', href);
        document.getElementsByTagName('head').item(0).appendChild(lnk);
    };

    var href = 'bower_components/tek-libraries/css/favorites.min.css';
    addLinkTagInHead(href, STYLESHEET_TYPE);
        
    this.showErrorMessage = function(response) {
        return MessageService.getMessageByCode('lib#UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (self.isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    return MessageService.getMessageByCode('lib#' + messageCode).then(function(message) {
                        return ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        return ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    if(!message.includes('ALE_SESSION_WILL_BE_ENDED')) {
                        return ScreenService.showMessage(message, 'ERROR');
                    }
                }
            } else {
                return ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    this.redirectToLogin = function() {
        if(window.openLoginContainerWithoutRedirect) {
            ScreenService.openWindow(self.authenticationContainer);
        } else {
            self.openWindow(self.authenticationContainer);
        }
    };

    eventAggregator.subscribe("EventOnRequestStart", function (eventData) {
        var validateExpiredSessionRoute = 'lib_validateExpiredSession';
        var logoutRoute = 'lib_logout';
        var loginRoute  = 'login';
        var validateUserRoute = 'validateUser';

        var requestConfig = eventData.requestConfig;
        if (requestConfig && requestConfig.url && shouldAddRequestInfo(requestConfig.url) && templateManager.container) {
            requestConfig.data.origin = requestConfig.data.origin || {};
            var widget;
            var container = templateManager.loadingContainerJson;
            if(container) {
                widget = getFirstWidgetWithDataSourceRest(container.widgets);
                if(!widget) {
                    widget = container.widgets[0];
                }
            } else {
                container = templateManager.container;
                widget = container.getActiveWidget();
            }

            requestConfig.data.origin.containerLabel = container.label || container.name;
            requestConfig.data.origin.widgetLabel = widget.label || widget.name;
        }

        var route = requestConfig.url.split('/').pop();
        if(route != validateExpiredSessionRoute) {
            factoryExpiredSessionMethods();
        }
        if(route != logoutRoute && route != loginRoute) {
            var token = localStorage.getItem('LOGIN_TOKEN');
            var hash = localStorage.getItem('LOGIN_HASH');
            var keepConnected = localStorage.getItem('LOGIN_KEEP_CONNECTED');
            if(window.externalTokens) {
                window.externalTokens.some(function(externalToken) {
                    if(eventData.requestConfig.url.includes(externalToken.url)) {
                        token = externalToken.token;
                        keepConnected = externalToken.keepConnected || keepConnected;
                        return true;
                    }
                    return false;
                });
            }
            if (token) {
                requestConfig.headers['OAuth-Token'] = token;
            }
            if (hash) {
                requestConfig.headers['OAuth-Hash'] = hash;
                requestConfig.headers['OAuth-Project'] = window.tekLoginProjectId;
            }
            if(route == validateUserRoute) {
                keepConnected = 'Yes';
            }
            requestConfig.headers['OAuth-KeepConnected'] = keepConnected;
        }
    });

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
            !url.match(/.html/i) &&
            !url.match(/.svg/i);
    };

    var getFirstWidgetWithDataSourceRest = function(widgets) {
        for(var i = 0; i < widgets.length; i++) {
            if(widgets[i].dataSource && widgets[i].dataSource.rest == true) {
                return widgets[i];
            } else {
                return getFirstWidgetWithDataSourceRest(widgets[i]);
            }
        }
    };

    var factoryExpiredSessionMethods = function() {
        if(templateManager.project && templateManager.project.onRequestIdleTimeout) {
            if(self.expiredSessionMethod) {
                clearTimeout(self.expiredSessionMethod);
            }
            self.expiredSessionMethod = Util.doAsync(validateExpiredSession, templateManager.project.onRequestIdleTimeout);
        }
    };

    var validateExpiredSession = function() {
        var timeout = templateManager.project.onRequestIdleTimeout;
        var filter = [ {
            name: 'TIMEOUT',
            value: timeout,
            operator: '='
        } ];

        requestDataSourceEngine.getDataSource('lib#/lib_validateExpiredSession', filter).then(function(response){
            if(response && response.dataset && response.dataset.expiredSessionInfo) {
                var expiredSessionInfo = response.dataset.expiredSessionInfo;
                if(expiredSessionInfo.IS_EXPIRED) {
                    eventAggregator.publish('LibExpiredSession');
                } else {
                    if(self.expiredSessionMethod) {
                        clearTimeout(self.expiredSessionMethod);
                    }
                    Util.doAsync(validateExpiredSession, expiredSessionInfo.TIMEOUT);
                }
            } else {
                self.showErrorMessage(response);
            }
        }).catch(self.showErrorMessage);
    };

    this.changeLoginResponseToOldParams = function(loginResponse) {
        if(loginResponse) {
            return {
                cdoperador:          loginResponse.USER_ID,
                nmoperador:          loginResponse.USER_NAME,
                dsemailoper:         loginResponse.EMAIL,
                nrorg:               loginResponse.ORGANIZATION_ID,
                nmorg:               loginResponse.TRADING_NAME,
                nmrazsocialorg:      loginResponse.COMPANY_NAME,
                perfilDeAcesso:      loginResponse.ACCESS_PROFILE,

                cdempresa:           loginResponse.CDEMPRESA     || null,
                cdfilialselecionada: loginResponse.CDFILIAL      || null,
                cdpictprat:          loginResponse.CDPICTPRAT    || null,
                cdpictprod:          loginResponse.CDPICTPROD    || null,
                ididioma:            loginResponse.IDIDIOMA      || null,
                nmacesso:            loginResponse.NMACESSO      || null,
                nmcomploper:         loginResponse.NMCOMPLOPER   || null,
                nmfilialselecionada: loginResponse.NMFILIAL      || null,
                nrestruttrab:        loginResponse.NRESTRUTTRAB  || null,
                nrorgtrab:           loginResponse.NRORGTRAB     || null,
                nrparcnegocio:       loginResponse.NRPARCNEGOCIO || null,

                logged:              loginResponse.LOGGED,
                senhaExpirada:       loginResponse.EXPIRED_PASSWORD,
                version:             loginResponse.VERSION
            };
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    this.openMicroServiceUrl = function(args) {
        var containerName = args.owner.msContainerName;
        var currentModule, module;

        if(containerName.indexOf('#') == -1) {
            currentModule = module = '';
        } else {
            var subModule = containerName.split('#')[0];
            module        = subModule.split('_')[0];
            currentModule = getCurrentModule();
        }

        if (module && currentModule && (module != currentModule)) {
            var filter = [ {
                name: 'USER',
                value: localStorage.getItem('LOGIN_USER'),
                operator: '='
            }, {
                name: 'IS_LOGGED',
                value: localStorage.getItem('IS_LOGGED') || false,
                operator: '='
            }, {
                name: 'KEEP_CONNECTED',
                value: localStorage.getItem('LOGIN_KEEP_CONNECTED') || 'No',
                operator: '='
            }, {
                name: 'LOGIN_LANGUAGE',
                value: localStorage.getItem('LOGIN_LANGUAGE') || window.lang,
                operator: '='
            }, {
                name: 'LOGIN_SEPARATOR',
                value: localStorage.getItem('LOGIN_SEPARATOR'),
                operator: '='
            } ];

            ScreenService.showLoader();
            requestDataSourceEngine.getDataSource(module + '#/lib_startSession', filter).then(function() {
                ScreenService.hideLoader();
                ScreenService.openLink(args.owner.url);
            }).catch(function(error) {
                ScreenService.hideLoader();
                self.showErrorMessage(error);
            });
        }
    };

    this.openLinkByMenu = function(menu) {
        if(typeof menu === 'string') {
            menu = templateManager.project.getMenu(menu);
        }

        var args = { owner: {
            msContainerName: menu.msContainerName,
            url: menu.url
        } };
        self.openMicroServiceUrl(args);
    };

    this.openWindow = function(windowName, newWindow) {
        return requestModulesMetadata().then(function (modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            if (ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            } else {
                ScreenService.openWindow(windowName);
            }
        });
    };

    var requestModulesMetadata = function () {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };

    eventAggregator.subscribe("AboutOnClick", function () {
        if(!templateManager.versionData) {
            templateManager.versionData = true;
            var filter = [ {
                name: 'CURRENT_MODULE',
                operator: '=',
                value: getCurrentModule()
            } ];
            ScreenService.showLoader();
            requestDataSourceEngine.getDataSource('lib#/lib_getVersionInfo', filter).then(function (response) {
                ScreenService.hideLoader();
                templateManager.versionData = response.dataset.versionInfo;
                templateManager.versionData.dependencies.current = templateManager.versionData.dependencies.frontend;
                templateManager.versionData.dependencies.changeCurrentVersion = function(newVersion) {
                    templateManager.versionData.dependencies.current = templateManager.versionData.dependencies[newVersion];
                };

                if (!!templateManager.versionData.dependencies) {
                    localStorage.setItem('versionData', JSON.stringify(templateManager.versionData));
                } else {
                    localStorage.setItem('versionData', JSON.stringify(false));
                }
            }).catch(function(error) {
                ScreenService.hideLoader();
                self.showErrorMessage(error);
            });
        }
    });

    eventAggregator.subscribe('onContainerOpen', function(args) {
        var versionData = JSON.parse(localStorage.getItem('versionData'));

        if (!args.container.isNext && 
             versionData && versionData.dependencies &&
             templateManager.versionData && templateManager.versionData.dependencies
            ) {
            templateManager.versionData.name = versionData.name;
            templateManager.versionData.version = versionData.version;
            templateManager.versionData.dependencies.backend = versionData.dependencies.backend;
            templateManager.versionData.dependencies.frontend = versionData.dependencies.frontend;
            templateManager.versionData.dependencies.modules = versionData.dependencies.modules;
            templateManager.versionData.dependencies.changeCurrentVersion('frontend');
            templateManager.updateTemplate();
        }
        menuFavoritos();
    });
    
    function menuFavoritos() {        
        // Aguarda renderização do componente de Acesso Rápido
        setTimeout(function() {            
            var groupId   = localStorage.getItem('ACCESS_PROFILE');
            var projectId = window.projectId;
            
            if(projectId && groupId) {
                var zhuAclFilter = ZhuAclConfig.getDefaultDataSourceFilter([groupId], projectId);
                // Busca o Controle de Acesso do Operador Logado
                requestDataSourceEngine.getDataSource('zhuacl#/acl_zhu_acl_rules', zhuAclFilter, 1, undefined, undefined, undefined, 120000, 120000, undefined, true).then(function(response) {
                    var dataset = response.dataset || {};
                    if(dataset) {                            
                        requestDataSourceEngine.getDataSource('lib#/lib_findFavorites', [], 1, undefined, undefined, undefined, 120000, 120000, undefined, true).then(function(result) {
                            var favorites = result.dataset.favorites;                            
                            favorites = favorites[0].map(function(item) {
                                return {
                                    id: item.ID,
                                    name: item.NAME,
                                    label: item.LABEL
                                };
                            });
                                        
                            if(favorites) {                                
                                // Verifica Controle de Acesso do Menu Favoritos
                                verificaAcessoFavoritos(dataset, favorites).then(function(favorites) {                        
                                    // Inclui Favoritos no componente de Acesso Rápido
                                    favorites = JSON.parse(JSON.stringify(favorites));
                                    eventAggregator.publish('ZhFavoritesOnLoad', favorites);
                                }.bind(this));    
                        
                            }                            
                        }, function(error) {});
                    }
                });
            }
        }, 2000);
    };
    
    function verificaAcessoFavoritos(dataset, favorites){
        var deferred = ZHPromise.defer();
        var pos = null;
        // Captura ID's dos Menus Favoritos
        var menuFavoritesIds = favorites.map(function(a) {return a.id;});
        // Regra Padrão
        var defaultRule = dataset.zhu_access_control_default_rule.defaultRule;
        
        // Preenche Regra Padrão para todos os Favoritos
        favorites.forEach(function(favorite){
            // Teste de Inativação
            favorite.rule = defaultRule;
        });
        
        // Percorre o Controle de Acesso dos Menus
        dataset.zhu_access_control_rule_login.forEach(function(acl){
            // Identifica existência de Controle de Acesso para o Menu
            if(menuFavoritesIds.includes(acl.UNIQUE_ID)){
                pos = menuFavoritesIds.indexOf(acl.UNIQUE_ID);
                // Preenche Regra para o Menu
                favorites[pos].rule = acl.RULE;
            }
        });
            
        // Permanece somente com os Favoritos ATIVOS(A)
        favorites = favorites.filter(function(favorite) {
            return ((favorite.rule == 'A') || (favorite.rule == 'R') || localStorage.getItem('ACL_ADMINISTRATOR'));
        });
        
        deferred.resolve(favorites);
        return deferred.promise;
    };
    
    eventAggregator.subscribe("ZhFavoritesOnChange", function (args) {
        /** 
          *Adiciona/Deleta Menu Favoritos
          * args.fullList = array<{ id, name, label }>
          * args.changed = { item: { id, name, label }, status: 'add'|'delete' }
        */
        args.changed.changed = true;
        requestDataSourceEngine.setDataSource('lib#/lib_changeFavorites', [args.changed], undefined, undefined, undefined, undefined, true);
    });

    eventAggregator.subscribe("EventOnRequestError", function (eventData) {
        if(eventData.e && eventData.e.e && eventData.e.e.data && eventData.e.e.data.data) {
            var error = eventData.e.e.data.data;
            var config = eventData.e.e.data.config;
            //Expired or Invalid Token
            if(error.errorCode == 101 || error.errorCode == 100)  {
                eventAggregator.publish('LibInvalidToken', error.errorCode);
            } else if(error.errorCode == 102){
                eventAggregator.publish('LibInvalidTokenSupportOperator');
            } else if(error.errorCode == 20){
                eventAggregator.publish('LibConcurrentAccess');
            }
        }
    });

    this.isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };

    /**
     * @param $collectionIds  => ID ou ARRAY da(s) coleta(s) (Cod Filial, Cod Operador, Cod da Loja, ...)
     * @param $operationId    => ID da operação (Retirada, Cardápio, Login, ...)
     * @param $productId      => Código do Produto (opcional).
     * @param $operatorCode   => Código do Operador (opcional).
     * @param $operatorName   => Nome do Operador (opcional).
     * @param $organizationId => Código do organização (opcional).
     */
    this.saveCollectedData = function(collectionIds, operationId, productId, operatorCode, operatorName, organizationId) {
        var row = {
            COLLECTION_IDS : collectionIds,
            OPERATION_ID   : operationId,
            PRODUCT_ID     : productId		|| null,
            OPERATOR_CODE  : operatorCode   || null,
            OPERATOR_NAME  : operatorName   || null,
            ORGANIZATION_ID: organizationId || null,
            changed        : true
        };

        return requestDataSourceEngine.setDataSource('lib#/lib_saveCollectedData', [row], undefined, undefined, undefined, undefined, true);
    };

    this.addTranslateButton = function() {
        templateManager.projectPromise.then(function() {
            if(!window.hideTranslateButton) {
                var notification = {
                    "label": ScreenService.i18n("Atualizar Tradução"),
                    "tooltip": ScreenService.i18n("Atualizar Tradução"),
                    "name": "libTranslate",
                    "icon": "information",
                    "isVisible": true,
                    "events": [ {
                        "name": "NotificationEvent",
                        "code": "LibLoginController.updateTranslation();"
                    } ]
                };
                notification = Util.extend(new _Notification(), notification);
                templateManager.project.notifications = templateManager.project.notifications || [];
                templateManager.project.notifications.push(notification);
            }
        });    
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibUtilities', LibUtilities);
});
